from pathlib import Path
import logging
from numpy import arange, array, ceil, empty, floor, isnan, linspace, \
    log10, meshgrid, nan, tile, transpose, where, hypot
from numpy.ma import masked_where
from matplotlib.pyplot import close, cm, colorbar, figure
try:
    from mpl_toolkits.basemap import Basemap
except ImportError:
    print('TODO: this code needs to be updated to cartopy')
    Basemap = None

try:
    import pyapex
except ImportError:
    pyapex = None

try:
    import pyigrf12
except ImportError:
    pyigrf12 = None

from scipy.interpolate import interp2d  # , RectBivariateSpline
#
from iri2016 import IRI
import iri2016
from timeutil import TimeUtilities
#
cwd = Path(__file__).parent
DataFolder = cwd / 'data'


class IRI2016_2DProf():

    # def __init__(self):

    #    pass

    # def _GetTitle(self):

    #    IRI2016Profile()._GetTitle(__self__)

    def HeightVsTime(self, FIRI=False, hrlim=[0., 24.], hrstp=1.):

        self.option = 1
        nhrstp = int((hrlim[1] + hrstp - hrlim[0]) / hrstp) + 1
        hrbins = list(map(lambda x: hrlim[0] + float(x) * hrstp, range(nhrstp)))

        Ne = empty((nhrstp, self.numstp))
        if FIRI:
            NeFIRI = empty((nhrstp, self.numstp))
        Te = empty((nhrstp, self.numstp))
        Ti = empty((nhrstp, self.numstp))

        for i in range(nhrstp):
            self.hour = hrbins[i]
            self.HeiProfile()

            Ne[i, :] = self.a[0, range(self.numstp)]
            if FIRI:
                NeFIRI[i, :] = self.a[12, range(self.numstp)]
            Te[i, :] = self.a[3, range(self.numstp)]
            Ti[i, :] = self.a[2, range(self.numstp)]

        # self._GetTitle()

        altbins = arange(self.vbeg, self.vend + self.vstp, self.vstp)
        self.data2D = {'alt': altbins, 'hour': hrbins,
                       'Ne': Ne, 'Te': Te, 'Ti': Ti,
                       'title1': self.title1, 'title2': self.title2}
        if FIRI:
            self.FIRI2D = {'alt': altbins, 'hour': hrbins,
                           'Ne': NeFIRI,
                           'title1': self.title1, 'title2': self.title2}

    def LatVsLon(self, lonlim=[-180., 180.], lonstp=20.):

        self.option = 2
        nlonstp = int((lonlim[1] + lonstp - lonlim[0]) / lonstp) + 1
        lonbins = list(map(lambda x: lonlim[0] + float(x) * lonstp, range(nlonstp)))

        NmF2 = empty((nlonstp, self.numstp))
        hmF2 = empty((nlonstp, self.numstp))
        B0 = empty((nlonstp, self.numstp))
        dip = empty((nlonstp, self.numstp))

        for i in range(nlonstp):
            self.lon = lonbins[i]
            self.HeiProfile()
            NmF2[i, :] = self.b[0, range(self.numstp)]
            hmF2[i, :] = self.b[1, range(self.numstp)]
            B0[i, :] = self.b[9, range(self.numstp)]
            dip[i, :] = self.b[24, range(self.numstp)]

        latbins = arange(self.vbeg, self.vend + self.vstp, self.vstp)
        self.data2D = {'lat': latbins, 'lon': lonbins,
                       'NmF2': NmF2, 'hmF2': hmF2, 'B0': B0, 'dip': dip,
                       'title': self.title3}

    def LatVsFL(self, date=[2003, 11, 21], FIRI=False, IGRF=False, time=[23, 15, 0],
                gc=[-77.76, -11.95],
                hlim=[80., 200.], hstp=1., mlatlim=[-10., 10.], mlatstp=.1):

        if pyapex is None:
            logging.error('PyApex is needed for LatVsFL')
            return
        # Date
        year, month, day = date

        # Time
        hour, minute, second = time

        # Geog. Coord.
        dlon, dlat = gc

        # hlim -> Height range at equator, in km
        # hstp -> height resolution at equator, in km
        # mlatlim -> Geom. latitude range, in degreesgetIGRF
        # mlatstp -> Geom. latitude resolution, in degrees

        #
        ###

        doy = TimeUtilities().CalcDOY(year, month, day)
        date2 = year + doy / (365 + 1 if TimeUtilities().IsLeapYear else 0)

        # f = figure(figsize=(16,6))

        # pn = f.add_subplot(111)

        self.coordl, self.qdcoordl = [], []

        for h in arange(hlim[0], hlim[1] + hstp, hstp):

            gc, qc = pyapex.ApexFL().getFL(date=date2, dlon=dlon, dlat=dlat,
                                           hateq=h, mlatRange=mlatlim, mlatSTP=mlatstp)

            # x, y, z = gc['lat'], gc['alt'], gc['lon']

            # ind = where(y < hlim[0])
            # if len(ind) > 0: x[ind], y[ind], z[ind] = nan, nan, nan
            # pn.plot(x, y)

            self.coordl.append([gc['lon'], gc['alt'], gc['lat']])
            self.qdcoordl.append([qc['lon'], gc['alt'], qc['lat']])

        # pn.invert_xaxis()
        # show()

        jf = IRI().Switches()
        jmag = 0
        mmdd = int(month * 100) + day
        hour2 = hour + minute / 60 + second / 3600

        self.coordl = array(self.coordl)
        self.qdcoordl = array(self.qdcoordl)

        # nfl -> No. of field-line (or height)
        # nc -> No. of coord. (0 -> lon, 1 -> alt, 2 -> lat)
        # np -> No. of points per field-line
        nfl, nc, np = self.coordl.shape

        self.ne, self.te = tile(nan, (np, nfl)), tile(nan, (np, nfl))
        self.ti, self.tn = tile(nan, (np, nfl)), tile(nan, (np, nfl))
        self.nHe, self.nO = tile(nan, (np, nfl)), tile(nan, (np, nfl))
        self.nN2, self.nO2 = tile(nan, (np, nfl)), tile(nan, (np, nfl))
        self.nAr, self.nH = tile(nan, (np, nfl)), tile(nan, (np, nfl))
        self.nN, self.babs = tile(nan, (np, nfl)), tile(nan, (np, nfl))
        if FIRI:
            self.neFIRI = tile(nan, (np, nfl))

        for fl in range(nfl):

            curr_coordl = transpose(self.coordl[fl, :, :])

            ind = where(curr_coordl[:, 1] >= (hlim[0] - 10.))

            if len(ind[0]) > 0:

                outf, oarr = iri2016.irisubgl(jf, jmag, year, mmdd, hour2,
                                              curr_coordl[ind[0], :], DataFolder)

                self.ne[ind[0], fl] = outf[0, :]

                self.tn[ind[0], fl] = outf[1, :]
                self.ti[ind[0], fl] = outf[2, :]
                self.te[ind[0], fl] = outf[3, :]

                if FIRI:
                    self.neFIRI[ind[0], fl], ierr = iri2016.firisubl(year, doy, hour2,
                                                                     curr_coordl[ind[0], :], DataFolder)

                self.nHe[ind[0], fl] = outf[20, :]
                self.nO[ind[0], fl] = outf[21, :]
                self.nN2[ind[0], fl] = outf[22, :]
                self.nO2[ind[0], fl] = outf[23, :]
                self.nAr[ind[0], fl] = outf[24, :]
                self.nH[ind[0], fl] = outf[26, :]
                self.nN[ind[0], fl] = outf[27, :]

                if IGRF:
                    self.babs[ind[0], fl] = list(self.getIGRF(curr_coordl[ind[0], :], date2))
                else:
                    self.babs[ind[0], fl] = outf[19, :]

        self.hlim = hlim

        self.date, self.time = date, time
        self.f107cm = oarr[40, 0]
        self.ap, self.Ap = oarr[50, 0], oarr[51, 0]

    def _Get_Title(self):

        dateStr = 'DATE: {:4d}/{:02d}/{:02d}'.format(self.date[0], self.date[1], self.date[2])
        timeStr = 'TIME: {:02d}:{:02d} UT'.format(self.time[0], self.time[1])
        f107Str = 'F107: {:6.2f}'.format(self.f107cm)
#        apStr = 'ap: {:3d}'.format(int(self.ap))
        ApStr = 'Ap: {:3d}'.format(int(self.Ap))
        gmlon = self.qdcoordl[0, 0, 0]
        gmlonStr = '{:7.2f} {:s}'.format(abs(gmlon), 'E' if gmlon > 0. else 'W')

        self._title1 = '{:s} - {:s}  -  MAG. LON.:{:s}'.format(dateStr, timeStr, gmlonStr)
        self._title2 = '{:s} - {:s}'.format(f107Str, ApStr)

    def getIGRF(self, coordl, yeardec):
        if pyigrf12 is None:
            logging.error('pyIGRF is not installed')
            return

        for glon, alt, glat in coordl:

            Bnorth, Beast, Bdown, Btotal = pyigrf12.runigrf12(yeardec, glat, glon, alt)

            # Horizontal component
            Bh = hypot(Bnorth, Beast)

            yield Bh

    def PlotLatVsFL(self):

        self._Get_Title()

        nrow, ncol = 2, 2

        spID = nrow * 100 + ncol * 10

        counter = 0

        X, Y = transpose(self.coordl[:, 2, :]), transpose(self.coordl[:, 1, :])

        f = figure(figsize=(16, 6))

        for ir in range(nrow):

            for ic in range(ncol):

                pn = f.add_subplot(spID + (counter + 1))

                if counter == 0:
                    Z = log10(self.ne)
                    vmin, vmax, nc = 8, 12, 32+1
                    zlabel = 'Log$_{10}$N$_e$(m$^{-3}$)'
                elif counter == 1:
                    Z = log10(self.nHe)
                    vmin, vmax, nc = 5, 9, 32+1
                    zlabel = 'Log$_{10}$H$_e$(m$^{-3}$)'
                elif counter == 2:
                    Z = self.te
                    vmin, vmax, nc = 100, 1200, 36+1
                    zlabel = r'T$_e$($^\circ$)'
                elif counter == 3:
                    Z = self.tn
                    vmin, vmax, nc = 100, 1200, 36+1
                    zlabel = r'T$_n$($^\circ$)'

                Z_masked = masked_where(isnan(Z), Z)

                C = linspace(vmin, vmax, nc, endpoint=True)
                ipc = pn.contourf(X, Y, Z_masked, C, cmap=cm.jet, extent='both', origin='lower')

                if counter == 0:
                    pn.set_title(self._title1)

                if counter == 1:
                    pn.set_title(self._title2)

                if counter > 1:
                    pn.set_xlabel(r'Geog. Lat. ($^\circ$)')

                pn.set_ylabel('Altitude (km)')
                pn.set_ylim(self.hlim)
                pn.invert_xaxis()
                pn.grid()

                cp = colorbar(ipc)
                cp.set_label(zlabel)

                counter += 1

    def PlotLatVsFLFIRI(self, save=False, verbose=False):

        self._Get_Title()

        nrow, ncol = 1, 1

        spID = nrow * 100 + ncol * 10

        counter = 0

        X, Y = transpose(self.coordl[:, 2, :]), transpose(self.coordl[:, 1, :])

        fg = figure(figsize=(16, 6))

        for ir in range(nrow):

            for ic in range(ncol):

                pn = fg.add_subplot(spID + (counter + 1))

                if counter == 0:
                    Z = log10(self.neFIRI)
                    vmin, vmax, nc = 9, 12, 24+1
                    zlabel = 'Log$_{10}$N$_e$(m$^{-3}$)'

                # Z_masked = masked_where(isnan(Z), Z)
                Z[where(Z < vmin)] = vmin

                C = linspace(vmin, vmax, nc, endpoint=True)
                ipc = pn.contourf(X, Y, Z, C, cmap=cm.jet, extent='both', origin='lower')

                if counter == 0:
                    pn.set_title(self._title1)
                # if counter == 1: pn.set_title(self._title2)
                pn.set_xlabel(r'Geog. Lat. ($^\circ$)')
                pn.set_ylabel('Altitude (km)')
                pn.set_ylim(self.hlim)
                pn.invert_xaxis()
                pn.grid()

                cp = colorbar(ipc)
                cp.set_label(zlabel)

                counter += 1

        if save:
            gpath = Path('../figures') / '{:04d}{:02d}{:02d}/'.format(self.year, self.month, self.dom)
            gpath.mkdir(parents=True, exist_ok=True)

            figname = gpath / 'firi-{:02d}{:02d}.jpg'.format(self.time[0], self.time[1])
            if verbose:
                print('Saving', figname)
            fg.savefig(figname, bbox_inches='tight', format='jpg', dpi=100)

            close(fg)

    def Plot2D(self, save=False):
        if Basemap is None:
            logging.error('TODO: this needs to be updated to cartopy')
            return

        fg = figure(figsize=(24, 6))

        if self.option == 1:

            pn = fg.add_subplot(131)
            X, Y = meshgrid(self.data2D['hour'], self.data2D['alt'])
            ipc = pn.pcolor(X, Y, transpose(log10(self.data2D['Ne'])), cmap=cm.jet, vmax=13, vmin=9)
            pn.set_title(self.data2D['title1'])
            pn.set_xlabel('Hour (UT)')
            pn.set_ylabel('Altitude (km)')
            cp1 = colorbar(ipc)
            cp1.set_label('Log$_{10}$N$_e$(m$^{-3}$)')

            pn = fg.add_subplot(132)
            ipc = pn.pcolor(X, Y, transpose(self.data2D['Te']), cmap=cm.jet, vmax=4000, vmin=100)
            pn.set_title(self.data2D['title2'])
            pn.set_xlabel('Hour (UT)')
            pn.set_ylabel('Altitude (km)')
            cp1 = colorbar(ipc)
            cp1.set_label(r'T$_e$ ($^\circ$)')

            pn = fg.add_subplot(133)
            ipc = pn.pcolor(X, Y, transpose(self.data2D['Ti']), cmap=cm.jet, vmax=4000, vmin=100)
            pn.set_xlabel('Hour (UT)')
            pn.set_ylabel('Altitude (km)')
            cp1 = colorbar(ipc)
            cp1.set_label(r'T$_i$ ($^\circ$)')

        elif self.option == 2:

            pn1 = fg.add_subplot(111)

            m = Basemap(llcrnrlon=self.data2D['lon'][0], llcrnrlat=self.data2D['lat'][0],
                        urcrnrlon=self.data2D['lon'][-1], urcrnrlat=self.data2D['lat'][-1],
                        resolution='l')
            m.drawcoastlines()

            parallelsLim = self._RoundLim([self.data2D['lat'][0], self.data2D['lat'][-1]])
            m.drawparallels(arange(parallelsLim[0], parallelsLim[1], 20.),
                            labels=[True, False, False, True])

            meridiansLim = self._RoundLim([self.data2D['lon'][0], self.data2D['lon'][-1]])
            m.drawmeridians(arange(meridiansLim[0], meridiansLim[1], 30.),
                            labels=[True, False, False, True])

            X, Y = meshgrid(self.data2D['lon'], self.data2D['lat'])
            ipc = m.pcolor(X, Y, transpose(9.*self.data2D['NmF2']**.5 * 1e-6),
                           cmap=cm.jet, vmax=15., vmin=0)
            m.contour(X, Y, transpose(self.data2D['dip']), colors='k', linestyles='--')
            pn1.set_title(self.data2D['title'])

            cp1 = m.colorbar(ipc)
            cp1.set_label('foF2 (MHz)')

        elif self.option == 8:
            pass

        if save:
            gpath = Path('../figures') / '{:04d}{:02d}{:02d}/'.format(self.year, self.month, self.dom)
            gpath.mkdir(parents=True, exist_ok=True)

            figname = gpath / 'iri-{:02d}{:02d}.jpg'.format(self.HH, self.MM)
            fg.savefig(figname, bbox_inches='tight', format='jpg', dpi=100)
            # convert -resize 50% -delay 20 -loop 0 *.jpg myimage.gif

    def PlotFIRI2D(self):

        f = figure(figsize=(8, 6))

        pn = f.add_subplot(111)

        if self.option == 1:

            X, Y = meshgrid(self.FIRI2D['hour'], self.FIRI2D['alt'])

            # ipc = pn.pcolor(X, Y, transpose(log10(self.FIRI2D['Ne'])), cmap=cm.jet,
            # vmax=12, vmin=9)

            Z = self.FIRI2D['Ne']
            Z[where(Z < 10**9)] = 10**9
            Z = transpose(log10(Z))

            C = linspace(9, 12, 24+1, endpoint=True)
            ipc = pn.contourf(X, Y, Z, C,
                              cmap=cm.jet, extent='both', origin='lower')
            pn.grid()
            pn.set_title(self.FIRI2D['title1'])
            pn.set_xlabel('Hour (UT)')
            pn.set_ylabel('Altitude (km)')

        elif self.option == 2:

            pass

        cp = colorbar(ipc)
        cp.set_label('Log$_{10}$N$_e$(m$^{-3}$)')

    def _RoundLim(self, lim):

        return list(map(lambda x: x * 10., [floor(lim[0] / 10.), ceil(lim[1] / 10.)]))

    def Plot2DMUF(self):

        f = figure(figsize=(16, 12))

        f.add_subplot(231)
        self.MapPColor(9.*self.data2D['NmF2']**.5 * 1e-6, 15., 5.)

        f.add_subplot(234)
        self.IntLatVsLon()
        self.MapPColorInt(self.data2DInt['foF2'], 15., 5.)

        f.add_subplot(232)
        self.MapPColor(self.data2D['hmF2'], 550., 250.)

        f.add_subplot(235)
        self.MapPColorInt(self.data2DInt['hmF2'], 550., 250.)

        f.add_subplot(233)
        self.MapPColor(self.data2D['B0'], 250., 100.)

        f.add_subplot(236)
        self.MapPColorInt(self.data2DInt['B0'], 250., 100.)

    def MapPColor(self, arr, vmax, vmin):

        self.m = Basemap(llcrnrlon=self.data2D['lon'][0], llcrnrlat=self.data2D['lat'][0],
                         urcrnrlon=self.data2D['lon'][-1], urcrnrlat=self.data2D['lat'][-1],
                         resolution='l')
        self.m.drawcoastlines()
        self.m.drawcountries()

        parallelsLim = self._RoundLim([self.data2D['lat'][0], self.data2D['lat'][-1]])
        self.m.drawparallels(arange(parallelsLim[0], parallelsLim[1], 2.),
                             labels=[True, False, False, True])

        meridiansLim = self._RoundLim([self.data2D['lon'][0], self.data2D['lon'][-1]])
        self.m.drawmeridians(arange(meridiansLim[0], meridiansLim[1], 5.),
                             labels=[True, False, False, True])

        X, Y = meshgrid(self.data2D['lon'], self.data2D['lat'])
#        ipc = self.m.pcolor(X, Y, transpose(arr), cmap=cm.jet, vmax=vmax, vmin=vmin)
        self.m.contour(X, Y, transpose(self.data2D['dip']), colors='k', linestyles='--')

        # self.m.plot(X, Y, color='k', linestyle='None', marker='o')

        # lon0, lat0 = -11.95, -76.87
        # x0, y0 = meshgrid(lon0, lat0)
        # self.m.plot(x0, y0, color='k', linestyle='None', marker='o')

        # print(x0, y0)

# ------------------------------------------------------------------------------

    def IntLatVsLon(self, lat0=-11.95, lon0=-76.87):

        # self.m.plot(lon0, lat0, 'bx')

        X0, Y0 = meshgrid(self.data2D['lon'], self.data2D['lat'])

        lon1 = lon0 + (array(self.data2D['lon']) - lon0) * .5
        lat1 = lat0 + (array(self.data2D['lat']) - lat0) * .5

        x0, y0 = array(self.data2D['lon']), array(self.data2D['lat'])

        foF2 = interp2d(x0, y0, 9.*transpose(self.data2D['NmF2'])**.5*1e-6)(lon1, lat1)
        hmF2 = interp2d(x0, y0, transpose(self.data2D['hmF2']))(lon1, lat1)
        B0 = interp2d(x0, y0, transpose(self.data2D['B0']))(lon1, lat1)

        self.data2DInt = {'lon': lon1, 'lat': lat1,
                          'foF2': transpose(foF2), 'hmF2': transpose(hmF2), 'B0': transpose(B0)}

        self.data2DTX = {}
        self.data2DTX['foF2'] = interp2d(x0, y0, 9.*transpose(self.data2D['NmF2'])**.5*1e-6)(lon0, lat0)[0]

    def MapPColorInt(self, arr, vmax, vmin):

        self.m = Basemap(llcrnrlon=self.data2D['lon'][0], llcrnrlat=self.data2D['lat'][0],
                         urcrnrlon=self.data2D['lon'][-1], urcrnrlat=self.data2D['lat'][-1],
                         resolution='l')
        self.m.drawcoastlines()
        self.m.drawcountries()

        parallelsLim = self._RoundLim([self.data2D['lat'][0], self.data2D['lat'][-1]])
        self.m.drawparallels(arange(parallelsLim[0], parallelsLim[1], 2.), labels=[True, False, False, True])

        meridiansLim = self._RoundLim([self.data2D['lon'][0], self.data2D['lon'][-1]])
        self.m.drawmeridians(arange(meridiansLim[0], meridiansLim[1], 5.), labels=[True, False, False, True])

        X, Y = meshgrid(self.data2DInt['lon'], self.data2DInt['lat'])
#        ipc = self.m.pcolor(X, Y, transpose(arr), cmap=cm.jet, vmax=vmax, vmin=vmin)

        X0, Y0 = meshgrid(self.data2D['lon'], self.data2D['lat'])
        self.m.contour(X0, Y0, transpose(self.data2D['dip']), colors='k', linestyles='--')
        # print(X.shape, Y.shape, arr.shape)



#!/usr/bin/env python
from pytest import approx
import pytest
from pathlib import Path
import iri2016

root = Path(__file__).parents[1]


def test_altitude_profile():
    time = '2015-12-13T10'
    altkmrange = (100, 1000, 10.)
    glat = 65.1
    glon = -147.5

    iri = iri2016.IRI(time, altkmrange, glat, glon)

# .item() necessary for stability across OS, pytest versions, etc.
    assert iri['ne'][10].item() == approx(3.98669824e9, rel=1e-4)
    assert iri.NmF2.item() == approx(7.71626844e+10, rel=1e-4)
    assert iri.hmF2.item() == approx(312.837677, rel=1e-4)


if __name__ == '__main__':
    pytest.main(['-xv', __file__])



#!/usr/bin/env python
import pytest
import subprocess


def test_latitude():
    pytest.importorskip('matplotlib')
    subprocess.check_call(['IRI16latitudeProfile', '-148'])


def test_time():
    pytest.importorskip('matplotlib')
    subprocess.check_call(['IRI16timeProfile', '65', '-148'])


def test_alt():
    pytest.importorskip('matplotlib')
    subprocess.check_call(['IRI16altitudeProfile', '65', '-148'])


if __name__ == '__main__':
    pytest.main(['-xv', __file__])



#!/usr/bin/env python
import setuptools  # noqa: F401
from pathlib import Path
# from numpy.distutils.core import Extension, setup
import os
import subprocess

"""
if os.name == 'nt':
    sfn = Path(__file__).parent / 'setup.cfg'
    stxt = sfn.read_text()
    if '[build_ext]' not in stxt:
        with sfn.open('a') as f:
            f.write("[build_ext]\ncompiler = mingw32")


src = [  # 'iriwebg.for',
    'irisub.for', 'irifun.for',
    'iritec.for', 'iridreg.for', 'igrf.for', 'cira.for', 'iriflip.for']

F = Path('src')
src = [str(F/s) for s in src]

ext = Extension(name='iri16', sources=src,
                f2py_options=['only:', 'iri_sub', ':'],  # ['skip:', 'dfridr', ':'],
                extra_f77_compile_args=['-w'])
"""

R = Path('iri2016') / 'data'
iridata = list(map(str,
                   (list((R/'ccir').glob('*.asc')) +
                    list((R/'igrf').glob('*.dat')) +
                       list((R/'index').glob('*.dat')) +
                       list((R/'mcsat').glob('*.dat')) +
                       list((R/'ursi').glob('*.asc')))
                   ))

iridata = list(map(str, iridata))  # even for Numpy 1.14 due to numpy.distutils

setuptools.setup(
    # ext_modules=[ext],
    data_files=iridata,
)

# %% workaround
if os.name == 'nt':
    subprocess.check_call(['cmake', '-G', 'MinGW Makefiles',
                           '-DCMAKE_SH="CMAKE_SH-NOTFOUND', '../src'],
                          cwd='bin',
                          universal_newlines=True)
else:
    subprocess.check_call(['cmake', '../src'], cwd='bin',
                          universal_newlines=True)


subprocess.check_call(['cmake', '--build', '.'], cwd='bin',
                      universal_newlines=True)



import xarray
from matplotlib.pyplot import figure


def timeprofile(iono: xarray.Dataset):
    # %% Plots
    Nplot = 3

    if Nplot > 2:
        fig = figure(figsize=(16, 12))
        axs = fig.subplots(Nplot, 1, sharex=True).ravel()
    else:
        fig = figure(figsize=(16, 6))
        axs = fig.subplots(1, 2).ravel()

    fig.suptitle(f'{str(iono.time[0].values)[:-13]} to '
                 f'{str(iono.time[-1].values)[:-13]}\n'
                 f'Glat, Glon: {iono.glat.item()}, {iono.glon.item()}')

    ax = axs[0]

    ax.plot(iono.time, iono['NmF2'], label='N$_m$F$_2$')
    ax.plot(iono.time, iono['NmF1'], label='N$_m$F$_1$')
    ax.plot(iono.time, iono['NmE'], label='N$_m$E')
    ax.set_title('Maximum number densities vs. ionospheric layer')
    ax.set_xlabel('Hour (UT)')
    ax.set_ylabel('(m$^{-3}$)')
    ax.set_yscale('log')
    ax.legend(loc='best')

    ax = axs[1]
    ax.plot(iono.time, iono['hmF2'], label='h$_m$F$_2$')
    ax.plot(iono.time, iono['hmF1'], label='h$_m$F$_1$')
    ax.plot(iono.time, iono['hmE'], label='h$_m$E')
    ax.set_title('Height of maximum density vs. ionospheric layer')
    ax.set_xlabel('Hour (UT)')
    ax.set_ylabel('(km)')
    ax.legend(loc='best')
    # %%
    if Nplot > 2:
        ax = axs[2]

        for a in iono.alt_km:
            ax.plot(iono.time, iono['ne'].sel(alt_km=a), marker='.', label=f'{a.item()} km')
        ax.set_xlabel('time UTC (hours)')
        ax.set_ylabel('[m$^{-3}$]')
        ax.set_title(f'$N_e$ vs. altitude and time')
        ax.set_yscale('log')
        ax.legend(loc='best')
    # %%
    if Nplot > 3:
        ax = axs[3]
        ax.plot(iono.time, iono['TEC'], label='TEC')
        ax.set_xlabel('Hour (UT)')
        ax.set_ylabel('(m$^{-2}$)')
        # ax.set_yscale('log')
        ax.legend(loc='best')
    if Nplot > 4:
        ax = axs[4]
        ax.plot(iono.time, iono['EqVertIonDrift'], label=r'V$_y$')
        ax.set_xlabel('Hour (UT)')
        ax.set_ylabel('(m/s)')
        ax.legend(loc='best')

    for a in axs.ravel():
        a.grid(True)


def altprofile(iono: xarray.Dataset):
    fig = figure(figsize=(16, 6))
    axs = fig.subplots(1, 2)

    fig.suptitle(f'{str(iono.time[0].values)[:-13]}\n'
                 f'Glat, Glon: {iono.glat.item()}, {iono.glon.item()}')

    pn = axs[0]
    pn.plot(iono['ne'], iono.alt_km, label='N$_e$')
    # pn.set_title(iri2016Obj.title1)
    pn.set_xlabel('Density (m$^{-3}$)')
    pn.set_ylabel('Altitude (km)')
    pn.set_xscale('log')
    pn.legend(loc='best')
    pn.grid(True)

    pn = axs[1]
    pn.plot(iono['Ti'], iono.alt_km, label='T$_i$')
    pn.plot(iono['Te'], iono.alt_km, label='T$_e$')
    # pn.set_title(iri2016Obj.title2)
    pn.set_xlabel('Temperature (K)')
    pn.set_ylabel('Altitude (km)')
    pn.legend(loc='best')
    pn.grid(True)


def latprofile(iono: xarray.Dataset):

    fig = figure(figsize=(8, 12))
    axs = fig.subplots(2, 1, sharex=True)

    ax = axs[0]

    ax.plot(iono['glat'], iono['NmF2'], label='N$_m$F$_2$')
    ax.plot(iono['glat'], iono['NmF1'], label='N$_m$F$_1$')
    ax.plot(iono['glat'], iono['NmE'], label='N$_m$E')
    ax.set_title(str(iono.time[0].values)[:-13] +
                 f'  latitude {iono["glat"][[0, -1]].values}')
    # ax.set_xlim(iono.lat[[0, -1]])
    ax.set_xlabel(r'Geog. Lat. ($^\circ$)')
    ax.set_ylabel('(m$^{-3}$)')
    ax.set_yscale('log')

    ax = axs[1]
    ax.plot(iono['glat'], iono['hmF2'], label='h$_m$F$_2$')
    ax.plot(iono['glat'], iono['hmF1'], label='h$_m$F$_1$')
    ax.plot(iono['glat'], iono['hmE'], label='h$_m$E')
    ax.set_xlim(iono['glat'][[0, -1]])
    ax.set_title(str(iono.time[0].values)[:-13] +
                 f'  latitude  {iono["glat"][[0, -1]].values}')
    ax.set_xlabel(r'Geog. Lat. ($^\circ$)')
    ax.set_ylabel('(km)')

    for a in axs:
        a.legend(loc='best')
        a.grid(True)



from pathlib import Path
from dateutil.parser import parse
import xarray
import numpy as np
#
import iri16  # fortran

proot = Path(__file__).parent
simout = ['ne', 'Tn', 'Ti', 'Te', 'nO+', 'nH+', 'nHe+', 'nO2+', 'nNO+',
          'nCI', 'nN+']


def Switches():
    """
     IRI switches to turn on/off several options
    """

    jf = np.ones(50, dtype=bool)

    #    i          1                       0                 standard version
    #    ------------------------------------------------------------------------
    jf[1 - 1] = 1  # 1    Ne computed            Ne not computed                     1
    jf[2 - 1] = 1  # 2    Te, Ti computed        Te, Ti not computed                 1
    jf[3 - 1] = 1  # 3    Ne & Ni computed       Ni not computed                     1
    jf[4 - 1] = 0  # 4    B0 - Table option      B0 - other models jf(31)            0
    jf[5 - 1] = 0  # 5    foF2 - CCIR            foF2 - URSI                         0
    jf[6 - 1] = 0  # 6    Ni - DS-95 & DY-85     Ni - RBV-10 & TTS-03                0
    jf[7 - 1] = 1  # 7    Ne - Tops: f10.7<188   f10.7 unlimited                     1
    jf[8 - 1] = 1  # 8    foF2 from model        foF2 or NmF2 - user input           1
    jf[9 - 1] = 1  # 9    hmF2 from model        hmF2 or M3000F2 - user input        1
    jf[10 - 1] = 1  # 10    Te - Standard          Te - Using Te/Ne correlation        1
    jf[11 - 1] = 1  # 11    Ne - Standard Profile  Ne - Lay-function formalism         1
    jf[12 - 1] = 1  # 12    Messages to unit 6     to meesages.text on unit 11         1
    jf[13 - 1] = 1  # 13    foF1 from model        foF1 or NmF1 - user input           1
    jf[14 - 1] = 1  # 14    hmF1 from model        hmF1 - user input (only Lay version)1
    jf[15 - 1] = 1  # 15    foE  from model        foE or NmE - user input             1
    jf[16 - 1] = 1  # 16    hmE  from model        hmE - user input                    1
    jf[17 - 1] = 1  # 17    Rz12 from file         Rz12 - user input                   1
    jf[18 - 1] = 1  # 18    IGRF dip, magbr, modip old FIELDG using POGO68/10 for 1973 1
    jf[19 - 1] = 1  # 19    F1 probability model   critical solar zenith angle (old)   1
    jf[20 - 1] = 1  # 20    standard F1            standard F1 plus L condition        1
    jf[21 - 1] = 1  # 21    ion drift computed     ion drift not computed              0
    jf[22 - 1] = 0  # 22    ion densities in %     ion densities in m-3                1
    jf[23 - 1] = 0  # 23    Te_tops (Aeros,ISIS)   Te_topside (TBT-2011)               0
    jf[24 - 1] = 1  # 24    D-region: IRI-95       Special: 3 D-region models (FIRI)   1
    jf[25 - 1] = 1  # 25    F107D from APF107.DAT  F107D user input (oarr(41))         1
    jf[26 - 1] = 0  # 26    foF2 storm model       no storm updating                   1
    jf[27 - 1] = 1  # 27    IG12 from file         IG12 - user                         1
    jf[28 - 1] = 0  # 28    spread-F probability   not computed                        0
    jf[29 - 1] = 0  # 29    IRI01-topside          new options as def. by JF(30)       0
    jf[30 - 1] = 0  # 30    IRI01-topside corr.    NeQuick topside model               0
    # (29,30) = (1,1) IRIold, (0,1) IRIcor, (0,0) NeQuick, (1,0) Gulyaeva
    jf[31 - 1] = 1  # 31    B0,B1 ABT-2009         B0 Gulyaeva h0.5                    1
    jf[32 - 1] = 1  # 32    F10.7_81 from file     PF10.7_81 - user input (oarr(46))   1
    jf[33 - 1] = 0  # 33    Auroral boundary model on/off  true/false                  0
    jf[34 - 1] = 0  # 34    Messages on            Messages off                        1
    jf[35 - 1] = 0  # 35    foE storm model        no foE storm updating               0
    #   ..    ....
    #   50    ....
    #   ------------------------------------------------------------------

    return jf


def IRI(time, altkmrange, glat, glon, ap=None, f107=None, ssn=None, var=None):

    if isinstance(time, str):
        time = parse(time)

    assert len(altkmrange) == 3, 'altitude (km) min, max, step'

#        doy = squeeze(TimeUtilities().CalcDOY(year, month, dom))

    # IRI options
    jf = Switches()

    # additional "input parameters" (necessary to scale the empirical model results
    # to measurements)
#    addinp = -np.ones(12)

    # ------------------------------------------------------------------------------
    #
#    if time.year < 1958:
#
#        addinp[10 - 1] = ssn  # RZ12  (This switches 'jf[17 - 1]' to '0' and
#                            # uses correlation function to estimate IG12)
#
#        jf[25 - 1] = 1        #   25    F107D from APF107.DAT  F107D user input (oarr(41))         1
#        jf[27 - 1] = 1        #   27    IG12 from file         IG12 - user                         1
#        jf[32 - 1] = 1        #   32    F10.7_81 from file     PF10.7_81 - user input (oarr(46))   1
#
#    else:  # case for solar and geomagnetic indices from files
#
#        jf[17 - 1] = 1        #   17    Rz12 from file         Rz12 - user input                   1
#        jf[26 - 1] = 1        #   26    foF2 storm model       no storm updating                   1
#        jf[35 - 1] = 1        #   35    foE storm model        no foE storm updating               0
#     #
#     #------------------------------------------------------------------------------

    mmdd = 100*time.month + time.day               # month and dom (MMDD)
    # hour + 25 denotes UTC time
    dhour = (time.hour + 25) + time.minute/60.
# %% more inputs
    jmag = 0  # 0: geographic; 1: geomagnetic
    # iut = 0             #  0: for LT;     1: for UT
    # height = 300.       #  in km
    # h_tec_max = 2000    #  0: no TEC; otherwise: upper boundary for integral
    # ivar = var          #  1: altitude; 2: latitude; 3: longitude; ...

    # Ionosphere (IRI)
#        a, b = iriwebg(jmag, jf, glat, glon, int(time.year), mmdd, iut, time.hour,
#            height, h_tec_max, ivar, ivbeg, ivend, ivstp, addinp, self.iriDataFolder)

    altkm = np.arange(*altkmrange)
    if altkm.size < 10:
        raise ValueError('Altitude grid must have enough points to compute quantities of interest')

    outf, oarr = iri16.iri_sub(jf, jmag, glat, glon,
                               time.year, mmdd, dhour,
                               altkmrange[0], altkmrange[1], altkmrange[2],
                               str(proot/'data/'))

    outf = outf[:, :altkm.size]
# %% collect output
    dsf = {k: (('time', 'alt_km', 'lat', 'lon'), np.atleast_2d(v[None, :, None, None])) for (k, v) in zip(simout, outf[:11, :])}

    dsf.update({'NmF2': (('time', 'lat', 'lon'), np.atleast_3d(oarr[0]))})
    dsf.update({'hmF2': (('time', 'lat', 'lon'), np.atleast_3d(oarr[1]))})
    dsf.update({'NmF1': (('time', 'lat', 'lon'), np.atleast_3d(oarr[2]))})
    dsf.update({'hmF1': (('time', 'lat', 'lon'), np.atleast_3d(oarr[3]))})
    dsf.update({'NmE': (('time', 'lat', 'lon'), np.atleast_3d(oarr[4]))})
    dsf.update({'hmE': (('time', 'lat', 'lon'), np.atleast_3d(oarr[5]))})
    dsf.update({'B0': (('time', 'lat', 'lon'), np.atleast_3d(oarr[9]))})

    iri = xarray.Dataset(dsf,
                         coords={'time': [time], 'alt_km': altkm, 'lat': [glat], 'lon': [glon]},
                         attrs={'f107': oarr[40], 'ap': oarr[50],
                                'glat': glat, 'glon': glon, 'time': time,
                                })

    return iri



import subprocess
from dateutil.parser import parse
from datetime import datetime, timedelta
from pathlib import Path
import os
import xarray
import io
import numpy as np
from typing import List, Sequence

R = Path(__file__).resolve().parents[1] / 'bin'
EXE = './iri2016_driver'
SHELL = False
if os.name == 'nt':
    EXE = EXE[2:] + '.exe'
    SHELL = True

SIMOUT = ['ne', 'Tn', 'Ti', 'Te', 'nO+', 'nH+', 'nHe+', 'nO2+', 'nNO+',
          'nCI', 'nN+']


def datetimerange(start: datetime, end: datetime, step: timedelta) -> List[datetime]:
    """like range() for datetime"""
    if isinstance(start, str):
        start = parse(start)

    if isinstance(end, str):
        end = parse(end)

    assert isinstance(start, datetime)
    assert isinstance(end, datetime)
    assert isinstance(step, timedelta)

    return [start + i*step for i in range((end-start) // step)]


def IRI(time: datetime, altkmrange: Sequence[float],
        glat: float, glon: float) -> xarray.Dataset:

    if isinstance(time, str):
        time = parse(time)

    assert len(altkmrange) == 3, 'altitude (km) min, max, step'
    assert isinstance(glat, float) and isinstance(glon, float), 'glat, glon is scalar'

    # NOTE: Windows needs shell=True and str(pathlib.Path)
    cmd = [str(EXE),
           str(time.year), str(time.month), str(time.day),
           str(time.hour), str(time.minute), str(time.second),
           str(glat), str(glon),
           str(altkmrange[0]), str(altkmrange[1]), str(altkmrange[2])]

    ret = subprocess.check_output(cmd,
                                  universal_newlines=True, cwd=str(R),
                                  shell=SHELL)
# %% get altitude profile data
    Nalt = int((altkmrange[1]-altkmrange[0]) // altkmrange[2]) + 1

    arr = np.genfromtxt(io.StringIO(ret), max_rows=Nalt)
    arr = np.atleast_2d(arr)
    assert arr.ndim == 2 and arr.shape[1] == 12, 'bad text data output format'

    dsf = {k: (('alt_km'), v) for (k, v) in zip(SIMOUT, arr[:, 1:].T)}
    altkm = arr[:, 0]
# %% get parameter data
    arr = np.genfromtxt(io.StringIO(ret), skip_header=Nalt)
    assert arr.ndim == 1 and arr.size == 100, 'bad text data output format'
# %% assemble output
    iono = xarray.Dataset(dsf,
                          coords={'time': [time], 'alt_km': altkm,
                                  'glat': glat, 'glon': glon},
                          attrs={'f107': arr[40], 'ap': arr[51]})

    for i, p in enumerate(['NmF2', 'hmF2', 'NmF1', 'hmF1', 'NmE', 'hmE']):
        iono[p] = (('time'), [arr[i]])

    iono['TEC'] = (('time'), [arr[36]])
    iono['EqVertIonDrift'] = (('time'), [arr[43]])

    return iono


def timeprofile(tlim: tuple, dt: timedelta,
                altkmrange: list, glat: float, glon: float) -> xarray.Dataset:
    """compute IRI altitude profile over time range for fixed lat/lon
    """

    T = datetimerange(tlim[0], tlim[1], dt)

    iono: xarray.Dataset = None

    f107 = []
    ap = []
    for t in T:
        iri = IRI(t, altkmrange, glat, glon)
        if iono is None:
            iono = iri
        else:
            iono = xarray.concat((iono, iri), dim='time')

        f107.append(iri.f107)
        ap.append(iri.ap)

    iono.attrs = iri.attrs
    iono.attrs['f107'] = f107
    iono.attrs['ap'] = ap

    return iono


def geoprofile(latrange: Sequence[float], glon: float,
               altkm: float,
               time: datetime) -> xarray.Dataset:
    """compute IRI altitude profiles at time, over lat or lon range
    """

    glat = np.arange(*latrange)

    iono: xarray.Dataset = None

    f107 = []
    ap = []
    for l in glat:
        iri = IRI(time, altkmrange=[altkm]*3, glat=l, glon=glon)
        if iono is None:
            iono = iri
        else:
            iono = xarray.concat((iono, iri), dim='glat')

        f107.append(iri.f107)
        ap.append(iri.ap)

    iono.attrs = iri.attrs
    iono.attrs['f107'] = f107
    iono.attrs['ap'] = ap

    return iono



#!/usr/bin/env python
from pathlib import Path
import subprocess
import pytest

R = Path(__file__).parent


def test_matlab_api():
    try:
        subprocess.check_call(['matlab', '-nojvm', '-r',
                               'r=runtests(); exit(any([r.Failed]))'],
                              cwd=R, timeout=60)
    except FileNotFoundError as e:
        pytest.skip(f'Matlab not available  {e}')


def test_octave_api():
    try:
        subprocess.check_call(['octave', '-q', '--eval="exit(test_iri2016)"'],
                              cwd=R, timeout=60)
    except FileNotFoundError as e:
        pytest.skip(f'GNU Octave not available  {e}')


if __name__ == '__main__':
    pytest.main(['-xrsv', __file__])
